@@ -6,7 +6,7 @@ import time
 import traceback
 import random
 import requests
-from datetime import datetime
+from datetime import datetime, timezone
 
 from MammothAnalytics import const
 from MammothAnalytics.const import USER_PERMISSIONS, RESERVED_BATCH_COLUMN_INTERNAL_NAMES_AND_KEYS
@@ -20,7 +20,7 @@ log = logging.getLogger(__name__)
 logging.basicConfig(level=logging.DEBUG)
 logging.getLogger("requests").setLevel(logging.WARNING)
 
-MAX_RETRIES = 50
+MAX_RETRIES = 80
 RETRY_DELAY_IN_SEC = 2
 
 
@@ -91,13 +91,15 @@ class MammothConnector(object):
                 raise AuthError(response.json()["ERROR_CODE"],
                                 '{0}'.format(response.status_code, response.json()['ERROR_MESSAGE']))
             self.__token = response.json()['token']
+            # adding account_id to the response object since the api key secret login returns the respective account id associated with the given key secret
+            account_id = response.json().get('account_id')
         else:
             raise AuthError("Token /Password /Appkey Appsecret needed!")
 
         if account_id is not None:
             self.select_account(account_id)
         else:
-            self.auto_select_account()
+            raise AuthError("Account ID needed!")
         log.info("Logged in as {0} into account {1}".format(email, self.__account_id))
 
     def __del__(self):
@@ -120,7 +122,8 @@ class MammothConnector(object):
             'get': requests.get,
             'post': requests.post,
             'delete': requests.delete,
-            'patch': requests.patch
+            'patch': requests.patch,
+            'put': requests.put
         }
         api_url = get_url(api, self.api_url)
         response = method_maps[rtype](api_url, **kwargs)
@@ -240,7 +243,9 @@ class MammothConnector(object):
     @handleError
     def delete_dataset(self, ds_id):
         response = self._make_signed_request('delete', '/datasets/{0}'.format(ds_id))
-        return response.json()
+        future_id = response.json()['future_id']
+        future_response = self.track_future_id(future_id)
+        return future_response
 
     @handleError
     def delete_identity(self, identity_key, integration_key):
@@ -310,11 +315,20 @@ class MammothConnector(object):
     @handleError
     def get_webhook_by_uri(self, webhook_uri):
         webhook_details = None
-        all_webhooks = self.list_webhooks()
-        for wh in all_webhooks:
-            if wh.get('url') == webhook_uri:
-                webhook_details = wh
+        retry_count = 0
+        while retry_count < MAX_RETRIES:
+            if retry_count != 0:
+                time.sleep(RETRY_DELAY_IN_SEC)
+            all_webhooks = self.list_webhooks()
+            for wh in all_webhooks:
+                if wh.get('url') == webhook_uri:
+                    webhook_details = wh
+                    break
+            if webhook_details is not None:
                 break
+            retry_count += 1
+        if retry_count == MAX_RETRIES:
+            raise RuntimeError(f"Max retry limit reached to get webhook details.")
         return webhook_details
 
     @handleError
@@ -327,7 +341,8 @@ class MammothConnector(object):
     def post_data_to_webhook(self, uri, data):
         if not isinstance(uri, str):
             raise TypeError("webhook uri should be of string type. found {0} instead".format(type(uri)))
-        return self._make_signed_request('post', uri, json=data)
+        response = self._make_signed_request('post', uri, json=data)
+        return response.json()
 
     @handleError
     def list_batches(self, ds_id):
@@ -450,15 +465,20 @@ class MammothConnector(object):
         return response.json()
 
     @handleError
-    def upload_csv(self, file_path, target_datasource_id=None, replace=False):
+    def upload_csv(self, file_path, target_datasource_id=None, replace=False, file_name=None,
+                   ds_with_no_header=False):
         """
         To upload an arbitrary csv file to the system. The system would return a file id based on which one can track
         the progress of the file through the mammoth system.
-
-        :param file_path: The file path
-        :param target_datasource_id: if this is set to a dataset id, the arbitrary csv would be appended to the
+        Args:
+            file_path - The file path
+            target_datasource_id - if this is set to a dataset id, the arbitrary csv would be appended to the
             data of the dataset.
-        :return: A file id.
+            replace - mode for which the file should be appended
+            file_name - file name for .xlsx, .xls, .zip
+            ds_with_no_header - if the given file has header or not
+        Return:
+            ds - A ds object.
         """
         files = {'file': open(file_path, 'rb')}
         post_data = {}
@@ -468,11 +488,33 @@ class MammothConnector(object):
                 post_data['replace'] = "true"
         response = self._make_signed_request('post', '/files', files=files, data=post_data)
         log.info(response.json())
-        return response.json()['id']
+        file_id = response.json()['id']
+        ds = self.wait_till_file_processing_get_ds(file_id, file_name, ds_with_no_header)
+        ds_id = ds['id']
+        if ds_with_no_header is False:
+            self.wait_till_ds_ready(ds_id)
+            views = self.list_views(ds_id)
+            if len(views) > 0:
+                view_ids = [view['id'] for view in views]
+                workspace_id = max(view_ids)
+                self.wait_for_view_to_finish_processing(workspace_id)
+            else:
+                retry_count = 0
+                while retry_count < MAX_RETRIES:
+                    if retry_count != 0:
+                        time.sleep(RETRY_DELAY_IN_SEC)
+                    views = self.list_views(ds_id)
+                    if len(views) > 0:
+                        view_ids = [view['id'] for view in views]
+                        workspace_id = max(view_ids)
+                        self.wait_for_view_to_finish_processing(workspace_id)
+                        break
+                    retry_count += 1
+        return ds
 
     @handleError
     def append_datasets(self, source_dataset_id, target_dataset_id, column_mapping, change_map=None,
-                        new_ds_params=None):
+                        new_ds_params=None, replace=False):
         """
         Method to combine two datasets While appending datasets, target ds gets combined and new rows gets added.At
         this point, ds updated_at will change. Thereafter, the pipeline re-run happens in all workspaces and
@@ -487,7 +529,8 @@ class MammothConnector(object):
         params = {
             "source": "datasource",
             "source_id": source_dataset_id,
-            "mapping": column_mapping
+            "mapping": column_mapping,
+            "replace": replace
         }
         if change_map:
             params.update({"change_map": change_map})
@@ -533,15 +576,17 @@ class MammothConnector(object):
             self.sync_now(target_dataset_id)
             time.sleep(15)
             r = self._make_signed_request('get', '/datasets/{0}/batches'.format(target_dataset_id))
-            result = r.json()['STATUS']
-            time.sleep(2)
-            if result == 'SUCCESS':
-                return response.json()
+            future_id = r.json().get('future_id')
+            future_response = self.track_future_id(future_id)
+            self.wait_till_ds_ready(target_dataset_id)
+            return future_response
 
-    def wait_till_file_processing_get_ds(self, file_id):
+    def wait_till_file_processing_get_ds(self, file_id, file_name=None, ds_with_no_header=False):
         """
-        Method will wait till the file with given id has finished processing. Works only for csv files right now.
-        TODO: Write logic to support excel, zip files.
+        Method will wait till the file with given id has finished processing.
+        Works for csv,txt,tsv files right now.
+        Also if we pass file_name to the method it works fine for .xslx, .xls
+        TODO: Write logic to support .zip file.
         :param file_id:
         :return: A dataset dictionary
         """
@@ -562,12 +607,24 @@ class MammothConnector(object):
         file_info = response.json()['file']
         final_ds_id = file_info["additional_data"].get("final_ds_id")
         append_to_ds_id = file_info["additional_data"].get("append_to_ds_id")
+        if final_ds_id is None and file_name is None:
+            raise UnknownError(f"File name required for .xls, .xlsx, .zip file extensions.")
+        # wait till the file reflects in resources to get dataset id by filtering resources
+        time.sleep(5)
+        if final_ds_id is None and file_name is not None:
+            resources = self.list_resources()
+            resources = list(filter(lambda x: x['resource_type'] == "datasource", resources))
+            for i in resources:
+                data = i['object_properties']['name']
+                if data == file_name:
+                    ds_id = i['object_properties']['id']
+                    final_ds_id = ds_id
         try:
             self._wait_for_ds_status(final_ds_id, 'processing', check_for_status_equality=False)
         except UnknownError as e:
             # in case there is a append to ds id, final ds may get deleted before status check
             log.error(e)
-        if append_to_ds_id:
+        if append_to_ds_id and ds_with_no_header is False:
             self._wait_for_ds_status(append_to_ds_id, 'ready')
             ds = self.get_dataset(append_to_ds_id)
         else:
@@ -595,10 +652,13 @@ class MammothConnector(object):
             raise RuntimeError(f"Max time limit reached in _wait_for_ds_status")
 
     @handleError
-    def sync_now(self, ds_id):
+    def sync_now(self, ds_id, wait_till_ready=False):
         r = self._make_signed_request('post', '/datasets/{}/apply-data'.format(ds_id))
         log.debug("The response of sync now i.e apply data is {}".format(r.json()))
-        return r.json()
+        response = r.json()
+        if wait_till_ready:
+            response = self.wait_till_batch_data_is_synced(ds_id)
+        return response
 
     @handleError
     def get_ds_for_file(self, file_id):
@@ -706,7 +766,7 @@ class MammothConnector(object):
             'offset': offset
         }
 
-        response = self._make_signed_request('get', 'workspaces/{workspace_id}/data'.format(workspace_id=view_id),
+        response = self._make_signed_request('post', 'workspaces/{workspace_id}/data'.format(workspace_id=view_id),
                                              json=paging_params)
         future_id = response.json()['future_id']
         future_response = self.track_future_id(future_id)
@@ -815,10 +875,10 @@ class MammothConnector(object):
 
         r = self._make_signed_request('patch', '/workspaces/{0}/actions/{1}'.format(view_id, action_id),
                                       json={"patch": [{"op": "replace", "path": "params", "value": param}]})
-        response = r.json()
+        response = r.json()['request_id']
         if wait_till_ready:
-            response = self._wait_till_request_is_complete_get_data(view_id, response['request_id'])
-        return response['request_id']
+            response = self._wait_till_request_is_complete_get_data(view_id, r.json()['request_id'])
+        return response
 
     def _wait_till_request_is_complete_get_data(self, view_id, request_id):
         retry_count = 0
@@ -967,9 +1027,11 @@ class MammothConnector(object):
         url = '/datasets'
         r = self._make_signed_request('post', url, json=ds_param)
         data = r.json()
-        ds_id = data['datasource_id']
+        # if the response json(data) contains datasource_id it returns ds_id otherwise returns none(only in case of file type dataset creation such as sftp, google drive, dropbox)
+        ds_id = data.get('datasource_id')
         if wait_till_ready:
-            self.wait_till_ds_ready(ds_id)
+            if ds_id is not None:
+                self.wait_till_ds_ready(ds_id)
         return ds_id
 
     def wait_till_ds_ready(self, ds_id):
@@ -1009,7 +1071,7 @@ class MammothConnector(object):
     def track_future_id(self, future_id):
         """
         Future request tracker that send GET request to Future API
-        and returns response as JSON object when status is not in 
+        and returns response as JSON object when status is not in
         processing (i.e. SUCCESS/FAILED)
         """
         if not isinstance(future_id, int):
@@ -1033,50 +1095,24 @@ class MammothConnector(object):
         return response
 
     @handleError
-    def add_explore_card(self, view_id, params):
-        """
-        Method to add explore card
-        Make post request to derivatives api, fetch derivative id,
-        Make get request to derivative api, fetch data of the derivative
-        :param view_id:
-        :type view_id:
-        :param params:
-        :type params:
-        :return:
-        :rtype:
-        """
-        response = self._make_signed_request('post', '/workspaces/{0}/derivatives'.format(view_id), json=params)
-        add_derivative_response = response.json()
-        log.debug("The add_derivative response id {}".format(add_derivative_response))
-        if add_derivative_response['STATUS'] == 'FAILURE':
-            return add_derivative_response
-        derivative_id = add_derivative_response['id']
-        query_response = {'STATUS': 'PROCESSING'}
-        retry_count = 0
-        data_response = query_response
-        while data_response['STATUS'] != 'READY' and retry_count < MAX_RETRIES:
-            response = self._make_signed_request('post', f'/workspaces/{view_id}/derivatives/{derivative_id}', json={})
-            data_response = response.json()
-            log.debug("Data response is {}".format(data_response))
-            if retry_count != 0:
-                time.sleep(RETRY_DELAY_IN_SEC)
-            retry_count += 1
-        if retry_count == MAX_RETRIES:
-            raise RuntimeError(f"Max limit reached to read query data.")
-
-        return data_response
-
-    @handleError
-    def add_batch_columns(self, ds_id, param):
+    def add_batch_columns(self, ds_id, param, wait_till_ready=True, ds_column_count=None):
         """
         Method to add batch columns to the Dataset
         :param ds_id:
         :param param - List of batch column/s to be added to the dataset
+        :param ds_column_count - expected column count for ds after batch column edition
         """
-        response = self._make_signed_request('patch', f'/datasets/{ds_id}',
+        res = self._make_signed_request('patch', f'/datasets/{ds_id}',
                                              json={"patch": [{"op": "replace", "path": "batch_columns",
                                                               "value": {"add_columns": param, "remove_columns": []}}]})
-        return response.json()
+        response = res.json()
+        if wait_till_ready:
+            future_id = response.get('future_id')
+            self.track_future_id(future_id)
+            self.wait_till_ds_ready(ds_id)
+            if ds_column_count is not None:
+                self.wait_till_batch_columns_are_updated_in_ds(ds_id, ds_column_count=ds_column_count)
+        return response
 
     @handleError
     def get_batch_column_keys(self, ds_id):
@@ -1095,21 +1131,29 @@ class MammothConnector(object):
         return batch_column_keys
 
     @handleError
-    def remove_batch_columns(self, ds_id, rem_cols):
+    def remove_batch_columns(self, ds_id, rem_cols, wait_till_ready=True, ds_column_count=None):
         """
         Method to remove batch columns from the Dataset
         :param ds_id:
         :param rem_cols - List of batch column/s (keys) to be removed from the dataset
+        :param ds_column_count - expected column count for ds after batch column deletion
         """
         # Get batch columns of the dataset
         batch_column_keys = self.get_batch_column_keys(ds_id)
         add_cols = list(set(batch_column_keys) - set(rem_cols))
 
-        response = self._make_signed_request('patch', '/datasets/{0}'.format(ds_id),
+        res = self._make_signed_request('patch', '/datasets/{0}'.format(ds_id),
                                              json={"patch": [{"op": "replace", "path": "batch_columns",
                                                               "value": {"add_columns": add_cols,
                                                                         "remove_columns": rem_cols}}]})
-        return response.json()
+        response = res.json()
+        if wait_till_ready:
+            future_id = response.get('future_id')
+            self.track_future_id(future_id)
+            self.wait_till_ds_ready(ds_id)
+            if ds_column_count is not None:
+                self.wait_till_batch_columns_are_updated_in_ds(ds_id, ds_column_count=ds_column_count)
+        return response
 
     @handleError
     def add_publish(self, view_id, params, wait_till_ready=True):
@@ -1228,3 +1272,461 @@ class MammothConnector(object):
                                                             "value": wksp_name}]})
         response = res.json()
         return response
+
+    @handleError
+    def apply_template(self, view_id, params):
+        """
+        method to apply template to the view
+        :param
+            view_id: workspace id
+            params: pipeline to be added
+        """
+        res = self._make_signed_request('post', 'workspaces/{0}/exportable-config'.format(view_id),
+                                        json={'config': params})
+        response = res.json()
+        return response
+
+    @handleError
+    def suspend_action(self, view_id, action_id):
+        """
+        method to suspend action
+        :param
+            view_id: workspace id
+            action_id: action id
+        """
+        r = self._make_signed_request('patch', '/workspaces/{0}/actions/{1}'.format(view_id, action_id),
+                                      json={"patch": [{"op": "replace", "path": "updateState", "value": "suspend"}]})
+        response = r.json()
+        return response
+
+    @handleError
+    def restore_action(self, view_id, action_id):
+        """
+        method to restore action
+        :param
+            view_id: workspace id
+            action_id: action id
+        """
+        r = self._make_signed_request('patch', '/workspaces/{0}/actions/{1}'.format(view_id, action_id),
+                                      json={"patch": [{"op": "replace", "path": "updateState", "value": "active"}]})
+        response = r.json()
+        return response
+
+    @handleError
+    def enable_sample_mode(self, view_id, wait_till_ready=True):
+        """
+        method to enable sample mode
+        :param
+            view_id: workspace id
+        """
+        response = self._make_signed_request('post', '/workspaces/{0}/preview-mode'.format(view_id),
+                                             json={"limit": 10000})
+        future_id = response.json()['future_id']
+        if wait_till_ready:
+            future_response = self.track_future_id(future_id)
+        return future_response
+
+    @handleError
+    def disable_sample_mode(self, view_id, wait_till_ready=True):
+        """
+        method to disable sample mode
+        :param
+            view_id: workspace id
+        """
+        response = self._make_signed_request('delete', '/workspaces/{0}/preview-mode'.format(view_id))
+        future_id = response.json()['future_id']
+        if wait_till_ready:
+            future_response = self.track_future_id(future_id)
+        return future_response
+
+    @handleError
+    def fix_schema_header(self, ds_id, params, wait_till_ready=True):
+        """
+        method to fix the uploaded file which needs user input (ex: csv file does not have header)
+        :param
+            ds_id: datasest id,
+            params: input params for the file to process
+        """
+        res = self._make_signed_request('post', '/datasets/{ds_id}/csvfile'.format(ds_id=ds_id), json=params)
+        response = res.json()
+        request_id = response.get('request_id')
+        if wait_till_ready:
+            response = self.track_async_request_id(request_id)
+        return response
+
+    @handleError
+    def track_async_request_id(self, request_id):
+        """
+        Async request tracker that send GET request to Async API
+        and returns response as JSON object when status is not in
+        processing (i.e. SUCCESS/FAILED)
+        """
+        if not isinstance(request_id, int):
+            raise RuntimeError(f"Request id should be a numeric value")
+
+        response = None
+        retry_count = 0
+        while retry_count < MAX_RETRIES:
+            if retry_count != 0:
+                time.sleep(RETRY_DELAY_IN_SEC)
+            resp_json = self._get_async_request_data(request_id)
+            if resp_json['STATUS'] != 'PROCESSING':
+                response = resp_json
+                break
+            retry_count += 1
+
+        if retry_count == MAX_RETRIES:
+            raise RuntimeError(f"Max limit reached to future request response.")
+        return response
+
+    @handleError
+    def wait_till_batch_is_updated(self, ds_id, batch_count=None):
+        """
+        Method to wait until the batch is updated
+        Args:
+            ds_id: datasest id
+            batch_count: expected task count in case of combined mode
+        Return:
+            batch_details: updated batch details
+        """
+        batch_details = None
+        # when the number of batches in the dataset is 0, we can not get the last_batch_date from the list batches api hence setting it to default value
+        last_batch_date = datetime.now(timezone.utc).replace(microsecond=0)
+        retry_count = 0
+        batches = self.list_batches(ds_id)
+        if len(batches) > 0:
+            last_batch_date = batches[-1]['created_at']
+            last_batch_date = datetime.strptime(last_batch_date, '%Y-%m-%dT%H:%M:%S%z')
+        while retry_count < MAX_RETRIES:
+            if retry_count != 0:
+                time.sleep(RETRY_DELAY_IN_SEC)
+            batches = self.list_batches(ds_id)
+            if batch_count is not None:
+                if len(batches) == batch_count:
+                    batch_details = batches
+                    break
+            else:
+                current_batch_date = batches[-1]['created_at']
+                current_batch_date = datetime.strptime(current_batch_date, '%Y-%m-%dT%H:%M:%S%z')
+                if not (current_batch_date < last_batch_date):
+                    batch_details = batches
+                    break
+            retry_count += 1
+        if retry_count == MAX_RETRIES:
+            raise RuntimeError(f"Max limit reached to update batch details.")
+        return batch_details
+
+    @handleError
+    def wait_till_task_details_updated(self, view_id, task_count):
+        """
+        Method to wait until the task details updated
+        Args:
+            view_id: workspace id
+            task_count: expected task count
+        Return:
+            task_details: updated task details
+        """
+        task_details = None
+        retry_count = 0
+        while retry_count < MAX_RETRIES:
+            if retry_count != 0:
+                time.sleep(RETRY_DELAY_IN_SEC)
+            tasks = self.get_task(view_id)
+            if len(tasks) == task_count:
+                task_details = tasks
+                break
+            retry_count += 1
+        if retry_count == MAX_RETRIES:
+            raise RuntimeError(f"Max limit reached to update task details.")
+        return task_details
+
+    @handleError
+    def wait_till_pipeline_status_is_updated(self, view_id, status):
+        """
+        Method to wait until the pipeline status changes
+        Args:
+            view_id: workspace id
+            status: expected status
+        Return:
+            pipeline_status: status of the pipeline
+        """
+        pipeline_status = None
+        retry_count = 0
+        while retry_count < MAX_RETRIES:
+            if retry_count != 0:
+                time.sleep(RETRY_DELAY_IN_SEC)
+            view_details = self.get_view_details(view_id)
+            if view_details['pipeline_status'] == status:
+                pipeline_status = view_details['pipeline_status']
+                break
+            retry_count += 1
+        if retry_count == MAX_RETRIES:
+            raise RuntimeError(f"Max limit reached to update pipeline status.")
+        return pipeline_status
+
+    @handleError
+    def wait_till_batch_columns_are_updated_in_ds(self, ds_id, ds_column_count):
+        """
+        Method to wait until the pipeline status changes
+        Args:
+            ds_id: dataset id
+            ds_column_count: expected column count for dataset
+        Return:
+            dataset_details: dataset details
+        """
+        dataset_details = None
+        retry_count = 0
+        while retry_count < MAX_RETRIES:
+            if retry_count != 0:
+                time.sleep(RETRY_DELAY_IN_SEC)
+            ds_details = self.get_dataset(ds_id)
+            if ds_details['column_count'] == ds_column_count:
+                dataset_details = ds_details
+                break
+            retry_count += 1
+        if retry_count == MAX_RETRIES:
+            raise RuntimeError(f"Max limit reached to add/edit batch columns.")
+        return dataset_details
+
+    @handleError
+    def wait_till_batch_data_is_synced(self, ds_id):
+        """
+        Method to wait until the added batch data is synced
+        Args:
+            ds_id: dataset id
+        Return:
+            dataset_details: dataset details
+        """
+        dataset_details = None
+        ds_row_count = 0
+        retry_count = 0
+
+        batches = self.list_batches(ds_id)
+        for batch in batches:
+            ds_row_count += batch['count']
+
+        while retry_count < MAX_RETRIES:
+            if retry_count != 0:
+                time.sleep(RETRY_DELAY_IN_SEC)
+            ds_details = self.get_dataset(ds_id)
+            if ds_details['row_count'] == ds_row_count:
+                dataset_details = ds_details
+                break
+            retry_count += 1
+        if retry_count == MAX_RETRIES:
+            raise RuntimeError(f"Max limit reached to sync dataset.")
+        return dataset_details
+
+    @handleError
+    def get_explore_data(self, view_id, params, wait_till_ready=True):
+        """
+        Method to add explore card data
+        Args:
+            view_id: workspace id
+            params: params for adding explore card
+        """
+        r = self._make_signed_request('post', '/workspaces/{0}/data/query'.format(view_id), json=params)
+        response = r.json()
+        if wait_till_ready:
+            response = self.track_future_id(response['future_id'])
+        return response
+
+    @handleError
+    def create_api_token(self, params):
+        """
+        Method to create api token
+        Args:
+            params: params for creating api token
+        """
+        response = self._make_signed_request('post', '/clientapps', json=params)
+        return response.json()
+
+    @handleError
+    def edit_api_token(self, params):
+        """
+        Method to edit api token
+        Args:
+            params: params for editing api token name and description
+        """
+        response = self._make_signed_request('put', '/clientapp', json=params)
+        return response.json()
+
+    @handleError
+    def delete_api_token(self, api_token_id):
+        """
+        Method to delete api token
+        Args:
+            api_token_id: id of the token to be deleted
+        """
+        response = self._make_signed_request('delete', '/clientapp?id=' + str(api_token_id))
+        return response.json()
+
+    @handleError
+    def list_api_tokens(self):
+        """
+        Method to list all the api tokens of an account
+        """
+        response = self._make_signed_request('get', '/clientapps')
+        return response.json()
+
+    @handleError
+    def change_user_role(self, account_id, user_id, user_role):
+        """
+        Method to change user role
+        Args:
+            account_id: id of the account
+            user_id: id of the user for which role to be changed
+            user_role: role to be assigned to the user
+        """
+        response = self._make_signed_request('patch', '/accounts/{0}/users/{1}'.format(account_id, user_id), json={
+            "patch": [{"op": "replace", "path": "user_permission", "value": user_role}]})
+        return response.json()
+
+    @handleError
+    def suspend_unsuspend_batches(self, ds_id, params, wait_till_ready=True):
+        """
+        Method to suspend/un-suspend batches
+        Args:
+            ds_id: dataset id
+            params: params for suspend/unsuspend batches
+        """
+        response = self._make_signed_request('patch', '/datasets/{0}/batches'.format(ds_id), json={
+            "patch": [{"op": "replace", "path": "batches", "value": params}]})
+        response = response.json()
+        if wait_till_ready:
+            future_id = response['future_id']
+            response = self.track_future_id(future_id)
+        return response
+
+    @handleError
+    def toggle_autosync(self, ds_id, auto_sync):
+        """
+        Method to toggle auto sync for a dataset
+        Args:
+            ds_id: dataset id
+            auto_sync: toggle value of auto sync on or off (True/False)
+        """
+        response = self._make_signed_request('patch', '/datasets/{0}'.format(ds_id), json={
+            "patch": [{"op": "replace", "path": "autosync", "value": auto_sync}]})
+        return response.json()
+
+    @handleError
+    def add_user(self, add_user_param):
+        """
+        Method to add user to an account
+        Args:
+            add_user_param: params for adding a user
+        """
+        response = self._make_signed_request('post', '/accounts/{0}/users'.format(self.__account_id),
+                                             json=add_user_param)
+        return response.json()
+
+    @handleError
+    def remove_user(self, user_id):
+        """
+        Method to remove user from an account
+        Args:
+            user_id: id of the user to be removed from account
+        """
+        response = self._make_signed_request('delete', '/accounts/{0}/users/{1}'.format(self.__account_id, user_id))
+        return response.json()
+
+    @handleError
+    def get_current_plan_details(self):
+        """
+        Method to get current plan details of account
+        Args:
+            NA
+        """
+        plan_details = {'message': 'This account is not associated with any plan'}
+        response = self._make_signed_request('get', '/accounts/{0}/sms-details'.format(self.__account_id))
+        current_plan_details = response.json().get('current_plan')
+        if current_plan_details is not None:
+            return current_plan_details
+        else:
+            return plan_details
+
+    @handleError
+    def get_user_app_usage(self, user_id):
+        """
+        Method to get app usage of the give user
+        Args:
+            user_id: user id
+        """
+        response = self._make_signed_request('get', '/accounts/{0}/app-usage/{1}'.format(self.__account_id, user_id))
+        return response.json()
+
+    @handleError
+    def edit_organization_name(self, organization_name):
+        """
+        Method to edit organization name
+        Args:
+            organization_name: name of the organization to be changed to
+        """
+        response = self._make_signed_request('patch', '/accounts/{0}'.format(self.__account_id), json={"patch": [{"op": "replace", "path": "name", "value": organization_name}]})
+        return response.json()
+
+    @handleError
+    def add_metric(self, view_id, params, condition=None, limit=None, wait_till_ready=True):
+        """
+        Method to add a metric
+        Args:
+            view_id: workspace id
+            param: param for adding a metric
+        """
+        res = self._make_signed_request('post', '/workspaces/{0}/derivatives'.format(view_id), json=params)
+        response = res.json()
+        if wait_till_ready:
+            response = self.wait_till_metric_is_ready(view_id, response.get('id'), condition, limit)
+        return response
+
+    @handleError
+    def delete_metric(self, view_id, metric_id, wait_till_ready=True):
+        """
+        Method to delete a metric
+        Args:
+            view_id: workspace id
+            metric_id: id of the metric
+        """
+        res = self._make_signed_request('delete', '/workspaces/{0}/derivatives/{1}'.format(view_id, metric_id))
+        response = res.json()
+        if wait_till_ready:
+            response = self._wait_till_request_is_complete_get_data(view_id, response.get('request_id'))
+        return response
+
+    @handleError
+    def list_metrics(self, view_id):
+        """
+        Method to list all the metrics of the given view
+        Args:
+            view_id: workspace id
+        """
+        res = self._make_signed_request('get', '/workspaces/{0}/derivatives'.format(view_id))
+        return res.json().get('derivatives')
+
+    @handleError
+    def wait_till_metric_is_ready(self, view_id, metric_id, condition=None, limit=None):
+        """
+        Method to wait until the metric is ready
+        Args:
+            view_id: workspace id
+            metric_id: id of the metric
+            condition: condition for the metric
+            limit: row limit
+        """
+        response = None
+        retry_count = 0
+        while retry_count < MAX_RETRIES:
+            if retry_count != 0:
+                time.sleep(RETRY_DELAY_IN_SEC)
+            metric_res = self._make_signed_request('post', '/workspaces/{0}/derivatives/{1}'.format(view_id, metric_id),
+                                                   json={"CONDITION": condition, "LIMIT": limit})
+            metric_resp_json = metric_res.json()
+            if metric_resp_json['STATUS'] == 'READY':
+                response = metric_resp_json
+                break
+            retry_count += 1
+
+        if retry_count == MAX_RETRIES:
+            raise RuntimeError(f"Max limit reached to add a metric")
+        return response
